from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PostApiPromptsByIdSyncResponse200PromptPromptingTechniqueDemonstrationsInlineColumnTypesItem")


@_attrs_define
class PostApiPromptsByIdSyncResponse200PromptPromptingTechniqueDemonstrationsInlineColumnTypesItem:
    """
    Attributes:
        name (str):
        type_ (Literal['annotations'] | Literal['boolean'] | Literal['chat_messages'] | Literal['date'] |
            Literal['evaluations'] | Literal['image'] | Literal['json'] | Literal['list'] | Literal['number'] |
            Literal['rag_contexts'] | Literal['spans'] | Literal['string']):
        id (str | Unset):
    """

    name: str
    type_: (
        Literal["annotations"]
        | Literal["boolean"]
        | Literal["chat_messages"]
        | Literal["date"]
        | Literal["evaluations"]
        | Literal["image"]
        | Literal["json"]
        | Literal["list"]
        | Literal["number"]
        | Literal["rag_contexts"]
        | Literal["spans"]
        | Literal["string"]
    )
    id: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        type_: (
            Literal["annotations"]
            | Literal["boolean"]
            | Literal["chat_messages"]
            | Literal["date"]
            | Literal["evaluations"]
            | Literal["image"]
            | Literal["json"]
            | Literal["list"]
            | Literal["number"]
            | Literal["rag_contexts"]
            | Literal["spans"]
            | Literal["string"]
        )
        type_ = self.type_

        id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "type": type_,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        def _parse_type_(
            data: object,
        ) -> (
            Literal["annotations"]
            | Literal["boolean"]
            | Literal["chat_messages"]
            | Literal["date"]
            | Literal["evaluations"]
            | Literal["image"]
            | Literal["json"]
            | Literal["list"]
            | Literal["number"]
            | Literal["rag_contexts"]
            | Literal["spans"]
            | Literal["string"]
        ):
            type_type_0 = cast(Literal["string"], data)
            if type_type_0 != "string":
                raise ValueError(f"type_type_0 must match const 'string', got '{type_type_0}'")
            return type_type_0
            type_type_1 = cast(Literal["boolean"], data)
            if type_type_1 != "boolean":
                raise ValueError(f"type_type_1 must match const 'boolean', got '{type_type_1}'")
            return type_type_1
            type_type_2 = cast(Literal["number"], data)
            if type_type_2 != "number":
                raise ValueError(f"type_type_2 must match const 'number', got '{type_type_2}'")
            return type_type_2
            type_type_3 = cast(Literal["date"], data)
            if type_type_3 != "date":
                raise ValueError(f"type_type_3 must match const 'date', got '{type_type_3}'")
            return type_type_3
            type_type_4 = cast(Literal["list"], data)
            if type_type_4 != "list":
                raise ValueError(f"type_type_4 must match const 'list', got '{type_type_4}'")
            return type_type_4
            type_type_5 = cast(Literal["json"], data)
            if type_type_5 != "json":
                raise ValueError(f"type_type_5 must match const 'json', got '{type_type_5}'")
            return type_type_5
            type_type_6 = cast(Literal["spans"], data)
            if type_type_6 != "spans":
                raise ValueError(f"type_type_6 must match const 'spans', got '{type_type_6}'")
            return type_type_6
            type_type_7 = cast(Literal["rag_contexts"], data)
            if type_type_7 != "rag_contexts":
                raise ValueError(f"type_type_7 must match const 'rag_contexts', got '{type_type_7}'")
            return type_type_7
            type_type_8 = cast(Literal["chat_messages"], data)
            if type_type_8 != "chat_messages":
                raise ValueError(f"type_type_8 must match const 'chat_messages', got '{type_type_8}'")
            return type_type_8
            type_type_9 = cast(Literal["annotations"], data)
            if type_type_9 != "annotations":
                raise ValueError(f"type_type_9 must match const 'annotations', got '{type_type_9}'")
            return type_type_9
            type_type_10 = cast(Literal["evaluations"], data)
            if type_type_10 != "evaluations":
                raise ValueError(f"type_type_10 must match const 'evaluations', got '{type_type_10}'")
            return type_type_10
            type_type_11 = cast(Literal["image"], data)
            if type_type_11 != "image":
                raise ValueError(f"type_type_11 must match const 'image', got '{type_type_11}'")
            return type_type_11

        type_ = _parse_type_(d.pop("type"))

        id = d.pop("id", UNSET)

        post_api_prompts_by_id_sync_response_200_prompt_prompting_technique_demonstrations_inline_column_types_item = (
            cls(
                name=name,
                type_=type_,
                id=id,
            )
        )

        post_api_prompts_by_id_sync_response_200_prompt_prompting_technique_demonstrations_inline_column_types_item.additional_properties = d
        return (
            post_api_prompts_by_id_sync_response_200_prompt_prompting_technique_demonstrations_inline_column_types_item
        )

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.post_api_analytics_timeseries_body_series_item_aggregation import (
    PostApiAnalyticsTimeseriesBodySeriesItemAggregation,
)
from ..models.post_api_analytics_timeseries_body_series_item_metric import (
    PostApiAnalyticsTimeseriesBodySeriesItemMetric,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.post_api_analytics_timeseries_body_series_item_filters import (
        PostApiAnalyticsTimeseriesBodySeriesItemFilters,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_pipeline import (
        PostApiAnalyticsTimeseriesBodySeriesItemPipeline,
    )


T = TypeVar("T", bound="PostApiAnalyticsTimeseriesBodySeriesItem")


@_attrs_define
class PostApiAnalyticsTimeseriesBodySeriesItem:
    """
    Attributes:
        metric (PostApiAnalyticsTimeseriesBodySeriesItemMetric):
        aggregation (PostApiAnalyticsTimeseriesBodySeriesItemAggregation):
        key (str | Unset):
        subkey (str | Unset):
        pipeline (PostApiAnalyticsTimeseriesBodySeriesItemPipeline | Unset):
        filters (PostApiAnalyticsTimeseriesBodySeriesItemFilters | Unset):
        as_percent (bool | Unset):
    """

    metric: PostApiAnalyticsTimeseriesBodySeriesItemMetric
    aggregation: PostApiAnalyticsTimeseriesBodySeriesItemAggregation
    key: str | Unset = UNSET
    subkey: str | Unset = UNSET
    pipeline: PostApiAnalyticsTimeseriesBodySeriesItemPipeline | Unset = UNSET
    filters: PostApiAnalyticsTimeseriesBodySeriesItemFilters | Unset = UNSET
    as_percent: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        metric = self.metric.value

        aggregation = self.aggregation.value

        key = self.key

        subkey = self.subkey

        pipeline: dict[str, Any] | Unset = UNSET
        if not isinstance(self.pipeline, Unset):
            pipeline = self.pipeline.to_dict()

        filters: dict[str, Any] | Unset = UNSET
        if not isinstance(self.filters, Unset):
            filters = self.filters.to_dict()

        as_percent = self.as_percent

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "metric": metric,
                "aggregation": aggregation,
            }
        )
        if key is not UNSET:
            field_dict["key"] = key
        if subkey is not UNSET:
            field_dict["subkey"] = subkey
        if pipeline is not UNSET:
            field_dict["pipeline"] = pipeline
        if filters is not UNSET:
            field_dict["filters"] = filters
        if as_percent is not UNSET:
            field_dict["asPercent"] = as_percent

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_analytics_timeseries_body_series_item_filters import (
            PostApiAnalyticsTimeseriesBodySeriesItemFilters,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_pipeline import (
            PostApiAnalyticsTimeseriesBodySeriesItemPipeline,
        )

        d = dict(src_dict)
        metric = PostApiAnalyticsTimeseriesBodySeriesItemMetric(d.pop("metric"))

        aggregation = PostApiAnalyticsTimeseriesBodySeriesItemAggregation(d.pop("aggregation"))

        key = d.pop("key", UNSET)

        subkey = d.pop("subkey", UNSET)

        _pipeline = d.pop("pipeline", UNSET)
        pipeline: PostApiAnalyticsTimeseriesBodySeriesItemPipeline | Unset
        if isinstance(_pipeline, Unset):
            pipeline = UNSET
        else:
            pipeline = PostApiAnalyticsTimeseriesBodySeriesItemPipeline.from_dict(_pipeline)

        _filters = d.pop("filters", UNSET)
        filters: PostApiAnalyticsTimeseriesBodySeriesItemFilters | Unset
        if isinstance(_filters, Unset):
            filters = UNSET
        else:
            filters = PostApiAnalyticsTimeseriesBodySeriesItemFilters.from_dict(_filters)

        as_percent = d.pop("asPercent", UNSET)

        post_api_analytics_timeseries_body_series_item = cls(
            metric=metric,
            aggregation=aggregation,
            key=key,
            subkey=subkey,
            pipeline=pipeline,
            filters=filters,
            as_percent=as_percent,
        )

        post_api_analytics_timeseries_body_series_item.additional_properties = d
        return post_api_analytics_timeseries_body_series_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.post_api_analytics_timeseries_body_group_by import PostApiAnalyticsTimeseriesBodyGroupBy
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.post_api_analytics_timeseries_body_filters import PostApiAnalyticsTimeseriesBodyFilters
    from ..models.post_api_analytics_timeseries_body_series_item import PostApiAnalyticsTimeseriesBodySeriesItem


T = TypeVar("T", bound="PostApiAnalyticsTimeseriesBody")


@_attrs_define
class PostApiAnalyticsTimeseriesBody:
    """
    Attributes:
        start_date (float | str):
        end_date (float | str):
        series (list[PostApiAnalyticsTimeseriesBodySeriesItem]):
        time_zone (str):
        query (str | Unset):
        filters (PostApiAnalyticsTimeseriesBodyFilters | Unset):
        trace_ids (list[str] | Unset):
        negate_filters (bool | Unset):
        group_by (PostApiAnalyticsTimeseriesBodyGroupBy | Unset):
        group_by_key (str | Unset):
        time_scale (int | Literal['full'] | Unset):
    """

    start_date: float | str
    end_date: float | str
    series: list[PostApiAnalyticsTimeseriesBodySeriesItem]
    time_zone: str
    query: str | Unset = UNSET
    filters: PostApiAnalyticsTimeseriesBodyFilters | Unset = UNSET
    trace_ids: list[str] | Unset = UNSET
    negate_filters: bool | Unset = UNSET
    group_by: PostApiAnalyticsTimeseriesBodyGroupBy | Unset = UNSET
    group_by_key: str | Unset = UNSET
    time_scale: int | Literal["full"] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        start_date: float | str
        start_date = self.start_date

        end_date: float | str
        end_date = self.end_date

        series = []
        for series_item_data in self.series:
            series_item = series_item_data.to_dict()
            series.append(series_item)

        time_zone = self.time_zone

        query = self.query

        filters: dict[str, Any] | Unset = UNSET
        if not isinstance(self.filters, Unset):
            filters = self.filters.to_dict()

        trace_ids: list[str] | Unset = UNSET
        if not isinstance(self.trace_ids, Unset):
            trace_ids = self.trace_ids

        negate_filters = self.negate_filters

        group_by: str | Unset = UNSET
        if not isinstance(self.group_by, Unset):
            group_by = self.group_by.value

        group_by_key = self.group_by_key

        time_scale: int | Literal["full"] | Unset
        if isinstance(self.time_scale, Unset):
            time_scale = UNSET
        else:
            time_scale = self.time_scale

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "startDate": start_date,
                "endDate": end_date,
                "series": series,
                "timeZone": time_zone,
            }
        )
        if query is not UNSET:
            field_dict["query"] = query
        if filters is not UNSET:
            field_dict["filters"] = filters
        if trace_ids is not UNSET:
            field_dict["traceIds"] = trace_ids
        if negate_filters is not UNSET:
            field_dict["negateFilters"] = negate_filters
        if group_by is not UNSET:
            field_dict["groupBy"] = group_by
        if group_by_key is not UNSET:
            field_dict["groupByKey"] = group_by_key
        if time_scale is not UNSET:
            field_dict["timeScale"] = time_scale

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_analytics_timeseries_body_filters import PostApiAnalyticsTimeseriesBodyFilters
        from ..models.post_api_analytics_timeseries_body_series_item import PostApiAnalyticsTimeseriesBodySeriesItem

        d = dict(src_dict)

        def _parse_start_date(data: object) -> float | str:
            return cast(float | str, data)

        start_date = _parse_start_date(d.pop("startDate"))

        def _parse_end_date(data: object) -> float | str:
            return cast(float | str, data)

        end_date = _parse_end_date(d.pop("endDate"))

        series = []
        _series = d.pop("series")
        for series_item_data in _series:
            series_item = PostApiAnalyticsTimeseriesBodySeriesItem.from_dict(series_item_data)

            series.append(series_item)

        time_zone = d.pop("timeZone")

        query = d.pop("query", UNSET)

        _filters = d.pop("filters", UNSET)
        filters: PostApiAnalyticsTimeseriesBodyFilters | Unset
        if isinstance(_filters, Unset):
            filters = UNSET
        else:
            filters = PostApiAnalyticsTimeseriesBodyFilters.from_dict(_filters)

        trace_ids = cast(list[str], d.pop("traceIds", UNSET))

        negate_filters = d.pop("negateFilters", UNSET)

        _group_by = d.pop("groupBy", UNSET)
        group_by: PostApiAnalyticsTimeseriesBodyGroupBy | Unset
        if isinstance(_group_by, Unset):
            group_by = UNSET
        else:
            group_by = PostApiAnalyticsTimeseriesBodyGroupBy(_group_by)

        group_by_key = d.pop("groupByKey", UNSET)

        def _parse_time_scale(data: object) -> int | Literal["full"] | Unset:
            if isinstance(data, Unset):
                return data
            time_scale_type_0 = cast(Literal["full"], data)
            if time_scale_type_0 != "full":
                raise ValueError(f"timeScale_type_0 must match const 'full', got '{time_scale_type_0}'")
            return time_scale_type_0
            return cast(int | Literal["full"] | Unset, data)

        time_scale = _parse_time_scale(d.pop("timeScale", UNSET))

        post_api_analytics_timeseries_body = cls(
            start_date=start_date,
            end_date=end_date,
            series=series,
            time_zone=time_zone,
            query=query,
            filters=filters,
            trace_ids=trace_ids,
            negate_filters=negate_filters,
            group_by=group_by,
            group_by_key=group_by_key,
            time_scale=time_scale,
        )

        post_api_analytics_timeseries_body.additional_properties = d
        return post_api_analytics_timeseries_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.post_api_traces_search_body_format import PostApiTracesSearchBodyFormat
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.post_api_traces_search_body_filters import PostApiTracesSearchBodyFilters


T = TypeVar("T", bound="PostApiTracesSearchBody")


@_attrs_define
class PostApiTracesSearchBody:
    """
    Attributes:
        start_date (float | str):
        end_date (float | str):
        query (str | Unset):
        filters (PostApiTracesSearchBodyFilters | Unset):
        trace_ids (list[str] | Unset):
        negate_filters (bool | Unset):
        page_offset (float | Unset):
        page_size (float | Unset):
        group_by (str | Unset):
        sort_by (str | Unset):
        sort_direction (str | Unset):
        updated_at (float | Unset):
        scroll_id (None | str | Unset):
        format_ (PostApiTracesSearchBodyFormat | Unset): Output format: 'digest' (AI-readable trace digest) or 'json'
            (full raw data)
        llm_mode (bool | Unset):
    """

    start_date: float | str
    end_date: float | str
    query: str | Unset = UNSET
    filters: PostApiTracesSearchBodyFilters | Unset = UNSET
    trace_ids: list[str] | Unset = UNSET
    negate_filters: bool | Unset = UNSET
    page_offset: float | Unset = UNSET
    page_size: float | Unset = UNSET
    group_by: str | Unset = UNSET
    sort_by: str | Unset = UNSET
    sort_direction: str | Unset = UNSET
    updated_at: float | Unset = UNSET
    scroll_id: None | str | Unset = UNSET
    format_: PostApiTracesSearchBodyFormat | Unset = UNSET
    llm_mode: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        start_date: float | str
        start_date = self.start_date

        end_date: float | str
        end_date = self.end_date

        query = self.query

        filters: dict[str, Any] | Unset = UNSET
        if not isinstance(self.filters, Unset):
            filters = self.filters.to_dict()

        trace_ids: list[str] | Unset = UNSET
        if not isinstance(self.trace_ids, Unset):
            trace_ids = self.trace_ids

        negate_filters = self.negate_filters

        page_offset = self.page_offset

        page_size = self.page_size

        group_by = self.group_by

        sort_by = self.sort_by

        sort_direction = self.sort_direction

        updated_at = self.updated_at

        scroll_id: None | str | Unset
        if isinstance(self.scroll_id, Unset):
            scroll_id = UNSET
        else:
            scroll_id = self.scroll_id

        format_: str | Unset = UNSET
        if not isinstance(self.format_, Unset):
            format_ = self.format_.value

        llm_mode = self.llm_mode

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "startDate": start_date,
                "endDate": end_date,
            }
        )
        if query is not UNSET:
            field_dict["query"] = query
        if filters is not UNSET:
            field_dict["filters"] = filters
        if trace_ids is not UNSET:
            field_dict["traceIds"] = trace_ids
        if negate_filters is not UNSET:
            field_dict["negateFilters"] = negate_filters
        if page_offset is not UNSET:
            field_dict["pageOffset"] = page_offset
        if page_size is not UNSET:
            field_dict["pageSize"] = page_size
        if group_by is not UNSET:
            field_dict["groupBy"] = group_by
        if sort_by is not UNSET:
            field_dict["sortBy"] = sort_by
        if sort_direction is not UNSET:
            field_dict["sortDirection"] = sort_direction
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if scroll_id is not UNSET:
            field_dict["scrollId"] = scroll_id
        if format_ is not UNSET:
            field_dict["format"] = format_
        if llm_mode is not UNSET:
            field_dict["llmMode"] = llm_mode

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_traces_search_body_filters import PostApiTracesSearchBodyFilters

        d = dict(src_dict)

        def _parse_start_date(data: object) -> float | str:
            return cast(float | str, data)

        start_date = _parse_start_date(d.pop("startDate"))

        def _parse_end_date(data: object) -> float | str:
            return cast(float | str, data)

        end_date = _parse_end_date(d.pop("endDate"))

        query = d.pop("query", UNSET)

        _filters = d.pop("filters", UNSET)
        filters: PostApiTracesSearchBodyFilters | Unset
        if isinstance(_filters, Unset):
            filters = UNSET
        else:
            filters = PostApiTracesSearchBodyFilters.from_dict(_filters)

        trace_ids = cast(list[str], d.pop("traceIds", UNSET))

        negate_filters = d.pop("negateFilters", UNSET)

        page_offset = d.pop("pageOffset", UNSET)

        page_size = d.pop("pageSize", UNSET)

        group_by = d.pop("groupBy", UNSET)

        sort_by = d.pop("sortBy", UNSET)

        sort_direction = d.pop("sortDirection", UNSET)

        updated_at = d.pop("updatedAt", UNSET)

        def _parse_scroll_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        scroll_id = _parse_scroll_id(d.pop("scrollId", UNSET))

        _format_ = d.pop("format", UNSET)
        format_: PostApiTracesSearchBodyFormat | Unset
        if isinstance(_format_, Unset):
            format_ = UNSET
        else:
            format_ = PostApiTracesSearchBodyFormat(_format_)

        llm_mode = d.pop("llmMode", UNSET)

        post_api_traces_search_body = cls(
            start_date=start_date,
            end_date=end_date,
            query=query,
            filters=filters,
            trace_ids=trace_ids,
            negate_filters=negate_filters,
            page_offset=page_offset,
            page_size=page_size,
            group_by=group_by,
            sort_by=sort_by,
            sort_direction=sort_direction,
            updated_at=updated_at,
            scroll_id=scroll_id,
            format_=format_,
            llm_mode=llm_mode,
        )

        post_api_traces_search_body.additional_properties = d
        return post_api_traces_search_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.get_evaluations_v3_run_status_response_200_status import GetEvaluationsV3RunStatusResponse200Status
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.get_evaluations_v3_run_status_response_200_summary import GetEvaluationsV3RunStatusResponse200Summary


T = TypeVar("T", bound="GetEvaluationsV3RunStatusResponse200")


@_attrs_define
class GetEvaluationsV3RunStatusResponse200:
    """
    Attributes:
        run_id (str):
        status (GetEvaluationsV3RunStatusResponse200Status):
        progress (int): Number of cells completed
        total (int): Total number of cells
        started_at (int | Unset): Unix timestamp when run started
        finished_at (int | Unset): Unix timestamp when run finished (only present when completed/failed/stopped)
        summary (GetEvaluationsV3RunStatusResponse200Summary | Unset): Execution summary (only present when completed)
        error (str | Unset): Error message (only present when failed)
    """

    run_id: str
    status: GetEvaluationsV3RunStatusResponse200Status
    progress: int
    total: int
    started_at: int | Unset = UNSET
    finished_at: int | Unset = UNSET
    summary: GetEvaluationsV3RunStatusResponse200Summary | Unset = UNSET
    error: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        run_id = self.run_id

        status = self.status.value

        progress = self.progress

        total = self.total

        started_at = self.started_at

        finished_at = self.finished_at

        summary: dict[str, Any] | Unset = UNSET
        if not isinstance(self.summary, Unset):
            summary = self.summary.to_dict()

        error = self.error

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "runId": run_id,
                "status": status,
                "progress": progress,
                "total": total,
            }
        )
        if started_at is not UNSET:
            field_dict["startedAt"] = started_at
        if finished_at is not UNSET:
            field_dict["finishedAt"] = finished_at
        if summary is not UNSET:
            field_dict["summary"] = summary
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_evaluations_v3_run_status_response_200_summary import (
            GetEvaluationsV3RunStatusResponse200Summary,
        )

        d = dict(src_dict)
        run_id = d.pop("runId")

        status = GetEvaluationsV3RunStatusResponse200Status(d.pop("status"))

        progress = d.pop("progress")

        total = d.pop("total")

        started_at = d.pop("startedAt", UNSET)

        finished_at = d.pop("finishedAt", UNSET)

        _summary = d.pop("summary", UNSET)
        summary: GetEvaluationsV3RunStatusResponse200Summary | Unset
        if isinstance(_summary, Unset):
            summary = UNSET
        else:
            summary = GetEvaluationsV3RunStatusResponse200Summary.from_dict(_summary)

        error = d.pop("error", UNSET)

        get_evaluations_v3_run_status_response_200 = cls(
            run_id=run_id,
            status=status,
            progress=progress,
            total=total,
            started_at=started_at,
            finished_at=finished_at,
            summary=summary,
            error=error,
        )

        get_evaluations_v3_run_status_response_200.additional_properties = d
        return get_evaluations_v3_run_status_response_200

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetEvaluationsV3RunStatusResponse200Summary")


@_attrs_define
class GetEvaluationsV3RunStatusResponse200Summary:
    """Execution summary (only present when completed)

    Attributes:
        run_id (str | Unset):
        total_cells (int | Unset):
        completed_cells (int | Unset):
        failed_cells (int | Unset):
        duration (int | Unset): Total execution time in milliseconds
        run_url (str | Unset): URL to view the run in LangWatch
    """

    run_id: str | Unset = UNSET
    total_cells: int | Unset = UNSET
    completed_cells: int | Unset = UNSET
    failed_cells: int | Unset = UNSET
    duration: int | Unset = UNSET
    run_url: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        run_id = self.run_id

        total_cells = self.total_cells

        completed_cells = self.completed_cells

        failed_cells = self.failed_cells

        duration = self.duration

        run_url = self.run_url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if run_id is not UNSET:
            field_dict["runId"] = run_id
        if total_cells is not UNSET:
            field_dict["totalCells"] = total_cells
        if completed_cells is not UNSET:
            field_dict["completedCells"] = completed_cells
        if failed_cells is not UNSET:
            field_dict["failedCells"] = failed_cells
        if duration is not UNSET:
            field_dict["duration"] = duration
        if run_url is not UNSET:
            field_dict["runUrl"] = run_url

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        run_id = d.pop("runId", UNSET)

        total_cells = d.pop("totalCells", UNSET)

        completed_cells = d.pop("completedCells", UNSET)

        failed_cells = d.pop("failedCells", UNSET)

        duration = d.pop("duration", UNSET)

        run_url = d.pop("runUrl", UNSET)

        get_evaluations_v3_run_status_response_200_summary = cls(
            run_id=run_id,
            total_cells=total_cells,
            completed_cells=completed_cells,
            failed_cells=failed_cells,
            duration=duration,
            run_url=run_url,
        )

        get_evaluations_v3_run_status_response_200_summary.additional_properties = d
        return get_evaluations_v3_run_status_response_200_summary

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
API-related types and structures for prompts.

Contains Pydantic models that match the API structure for messages,
inputs, outputs, and response formats.
"""

from typing import Literal, Optional, Dict, Any

from pydantic import BaseModel


class Message(BaseModel):
    """Message model that matches the API structure."""

    role: Literal["system", "user", "assistant"]
    content: str


class Input(BaseModel):
    """Input model that matches the API structure."""

    identifier: str
    type: Literal["str", "int", "float", "bool", "json"]


class Output(BaseModel):
    """Output model that matches the API structure."""

    identifier: str
    type: Literal["str", "int", "float", "bool", "json"]
    json_schema: Optional[Dict[str, Any]] = None


class ResponseFormat(BaseModel):
    """Response format model for structured outputs."""

    type: Literal["json_schema"]
    json_schema: Optional[Dict[str, Any]] = None


# Backward compatibility aliases
MessageDict = Message
InputDict = Input
OutputDict = Output
ResponseFormatDict = ResponseFormat



"""
Unit tests for target() context manager in Evaluation class.
"""

import pytest
import time
from unittest.mock import patch, MagicMock
from langwatch.experiment.experiment import (
    Experiment,
    _target_context,
)


@pytest.fixture
def evaluation():
    """Create an evaluation that won't send batches during tests."""
    ev = Experiment("test-experiment")
    ev.initialized = True
    ev._current_index = 0
    ev._current_item = {"question": "test"}
    # Set last_sent to far future to prevent automatic sending
    ev.last_sent = time.time() + 100000
    return ev


class TestTargetBasics:
    """Basic functionality tests for target()."""

    def test_target_creates_dataset_entry(self, evaluation):
        """target() creates a dataset entry with target_id."""
        evaluation._current_item = {"question": "What is AI?"}

        with evaluation.target("gpt-4", {"model": "openai/gpt-4"}):
            pass  # Just execute something

        # Should have one dataset entry with target_id
        assert len(evaluation.batch["dataset"]) == 1
        entry = evaluation.batch["dataset"][0]
        assert entry.target_id == "gpt-4"
        assert entry.index == 0

    def test_target_captures_duration(self, evaluation):
        """target() captures duration automatically."""
        with evaluation.target("gpt-4"):
            time.sleep(0.05)  # 50ms

        entry = evaluation.batch["dataset"][0]
        assert entry.duration >= 50  # At least 50ms

    def test_target_registers_target_with_metadata(self, evaluation):
        """target() registers the target with metadata."""
        with evaluation.target("gpt-4", {"model": "openai/gpt-4", "temp": 0.7}):
            pass

        assert "gpt-4" in evaluation._targets
        assert evaluation._targets["gpt-4"].metadata == {"model": "openai/gpt-4", "temp": 0.7}

    def test_target_stores_entry_data(self, evaluation):
        """target() stores the current item in the dataset entry."""
        evaluation._current_item = {"question": "What is AI?", "expected": "answer"}

        with evaluation.target("gpt-4"):
            pass

        entry = evaluation.batch["dataset"][0]
        # Dict items without to_dict() or __dict__ are serialized as JSON
        assert "entry" in entry.entry
        assert "What is AI?" in entry.entry["entry"]


class TestMultipleTargets:
    """Tests for using multiple targets in one iteration."""

    def test_multiple_targets_create_multiple_entries(self, evaluation):
        """Multiple target() calls create multiple dataset entries."""
        with evaluation.target("gpt-4", {"model": "openai/gpt-4"}):
            pass

        with evaluation.target("claude", {"model": "anthropic/claude"}):
            pass

        # Should have 2 dataset entries
        assert len(evaluation.batch["dataset"]) == 2

        target_ids = [e.target_id for e in evaluation.batch["dataset"]]
        assert "gpt-4" in target_ids
        assert "claude" in target_ids

    def test_targets_have_independent_durations(self, evaluation):
        """Each target captures its own duration."""
        with evaluation.target("fast-model"):
            time.sleep(0.02)  # 20ms

        with evaluation.target("slow-model"):
            time.sleep(0.05)  # 50ms

        fast_entry = next(e for e in evaluation.batch["dataset"] if e.target_id == "fast-model")
        slow_entry = next(e for e in evaluation.batch["dataset"] if e.target_id == "slow-model")

        # Slow should be slower than fast
        assert slow_entry.duration > fast_entry.duration


class TestContextInference:
    """Tests for target context inference in log() calls."""

    def test_log_infers_target_from_context(self, evaluation):
        """log() infers target from target() context."""
        with evaluation.target("gpt-4"):
            # Log without explicit target - should use context
            evaluation.log("quality", index=0, score=0.95)

        # Check the evaluation has the target_id from context
        assert len(evaluation.batch["evaluations"]) == 1
        assert evaluation.batch["evaluations"][0].target_id == "gpt-4"

    def test_log_explicit_target_overrides_context(self, evaluation):
        """log() with explicit target overrides context."""
        with evaluation.target("gpt-4"):
            # Log with explicit different target
            evaluation.log("quality", index=0, score=0.95, target="custom-target")

        assert evaluation.batch["evaluations"][0].target_id == "custom-target"

    def test_context_is_reset_after_target(self, evaluation):
        """Context is properly reset after exiting target()."""
        with evaluation.target("gpt-4"):
            # Inside context
            ctx = _target_context.get()
            assert ctx is not None
            assert ctx.target_id == "gpt-4"

        # Outside context
        ctx = _target_context.get()
        assert ctx is None

    def test_log_with_data_works_inside_target(self, evaluation):
        """log() with data= parameter works correctly inside target()."""
        with evaluation.target("gpt-4"):
            evaluation.log(
                "quality",
                index=0,
                score=0.95,
                data={"output": "AI response", "tokens": 150}
            )

        eval_result = evaluation.batch["evaluations"][0]
        assert eval_result.target_id == "gpt-4"
        assert eval_result.data == {"output": "AI response", "tokens": 150}


class TestRowLevelEntryPrevention:
    """Tests for preventing duplicate row-level dataset entries."""

    def test_target_sets_flag(self, evaluation):
        """target() sets _current_iteration_used_with_target flag."""
        assert evaluation._current_iteration_used_with_target is False

        with evaluation.target("gpt-4"):
            assert evaluation._current_iteration_used_with_target is True

    def test_flag_prevents_row_level_entry_in_loop(self, evaluation):
        """When flag is set, only target-specific entries exist."""
        # Use target - this sets the flag
        with evaluation.target("gpt-4"):
            pass

        # Only the target entry should exist, not a row-level one
        assert len(evaluation.batch["dataset"]) == 1
        assert evaluation.batch["dataset"][0].target_id == "gpt-4"


class TestErrorHandling:
    """Tests for error handling in target()."""

    def test_error_is_captured_in_entry(self, evaluation):
        """Errors inside target() are captured in the dataset entry."""
        with pytest.raises(ValueError):
            with evaluation.target("gpt-4"):
                raise ValueError("Test error")

        # Entry should still be created with error
        assert len(evaluation.batch["dataset"]) == 1
        assert evaluation.batch["dataset"][0].error == "Test error"

    def test_error_is_reraised(self, evaluation):
        """Errors are re-raised after cleanup."""
        with pytest.raises(ValueError, match="Test error"):
            with evaluation.target("gpt-4"):
                raise ValueError("Test error")

    def test_context_reset_on_error(self, evaluation):
        """Context is properly reset even when error occurs."""
        try:
            with evaluation.target("gpt-4"):
                raise ValueError("Test error")
        except ValueError:
            pass

        # Context should be reset
        assert _target_context.get() is None


class TestSimpleUsage:
    """Tests for the simplified API without result assignment."""

    def test_simple_target_usage(self, evaluation):
        """Simple usage without result assignment works."""
        with evaluation.target("gpt-4", {"model": "openai/gpt-4"}):
            # Simulate calling an LLM
            time.sleep(0.01)  # Add small delay for duration
            response = "This is the AI response"
            # Log the response and a metric
            evaluation.log_response(response)
            evaluation.log("quality", index=0, score=0.95)

        # Should have dataset entry with predicted
        assert len(evaluation.batch["dataset"]) == 1
        assert evaluation.batch["dataset"][0].target_id == "gpt-4"
        assert evaluation.batch["dataset"][0].duration >= 10  # At least 10ms
        assert evaluation.batch["dataset"][0].predicted == {"output": "This is the AI response"}

        # Should have evaluation
        assert len(evaluation.batch["evaluations"]) == 1

    def test_no_as_clause_needed(self, evaluation):
        """target() works without 'as' clause."""
        # This should work without "as result:"
        with evaluation.target("gpt-4"):
            evaluation.log("metric", index=0, score=1.0)

        assert len(evaluation.batch["dataset"]) == 1
        assert len(evaluation.batch["evaluations"]) == 1


class TestLogResponse:
    """Tests for log_response() method."""

    def test_log_response_string(self, evaluation):
        """log_response() with string wraps as {"output": ...}."""
        with evaluation.target("gpt-4"):
            evaluation.log_response("Hello, world!")

        assert evaluation.batch["dataset"][0].predicted == {"output": "Hello, world!"}

    def test_log_response_dict(self, evaluation):
        """log_response() with dict uses it as-is."""
        with evaluation.target("gpt-4"):
            evaluation.log_response({"answer": "42", "confidence": 0.95})

        assert evaluation.batch["dataset"][0].predicted == {"answer": "42", "confidence": 0.95}

    def test_log_response_outside_target_creates_implicit_output_target(self, evaluation):
        """log_response() outside target() creates an implicit 'Output' target."""
        # Set up iteration context (normally done by loop())
        from langwatch.experiment.experiment import _iteration_context, _target_context, IterationContext
        iter_ctx = IterationContext(index=0, item={"question": "test"})
        iter_token = _iteration_context.set(iter_ctx)

        try:
            evaluation.log_response("Hello, world!")

            # Should create a dataset entry with target_id "Output"
            assert len(evaluation.batch["dataset"]) == 1
            entry = evaluation.batch["dataset"][0]
            assert entry.target_id == "Output"
            assert entry.predicted == {"output": "Hello, world!"}
            assert entry.index == 0

            # Should register the "Output" target
            assert "Output" in evaluation._targets
        finally:
            _iteration_context.reset(iter_token)
            # Reset target context to avoid polluting other tests
            _target_context.set(None)

    def test_log_response_outside_target_associates_log_with_output_target(self, evaluation):
        """log() after log_response() outside target uses the implicit 'Output' target."""
        from langwatch.experiment.experiment import _iteration_context, _target_context, IterationContext
        iter_ctx = IterationContext(index=0, item={"question": "test"})
        iter_token = _iteration_context.set(iter_ctx)

        try:
            evaluation.log_response("Hello, world!")
            evaluation.log("quality", index=0, score=0.95)

            # log() should use the implicit "Output" target
            assert len(evaluation.batch["evaluations"]) == 1
            assert evaluation.batch["evaluations"][0].target_id == "Output"
        finally:
            _iteration_context.reset(iter_token)
            # Reset target context to avoid polluting other tests
            _target_context.set(None)

    def test_log_response_with_multiple_metrics(self, evaluation):
        """log_response() works alongside multiple log() calls."""
        with evaluation.target("gpt-4"):
            evaluation.log_response("AI response")
            evaluation.log("accuracy", index=0, score=0.95)
            evaluation.log("latency", index=0, score=150)
            evaluation.log("cost", index=0, score=0.001)

        # One dataset entry with predicted
        assert len(evaluation.batch["dataset"]) == 1
        assert evaluation.batch["dataset"][0].predicted == {"output": "AI response"}

        # Three evaluations
        assert len(evaluation.batch["evaluations"]) == 3


class TestImplicitTargetContextReset:
    """Tests to ensure implicit target context is properly reset between iterations."""

    def test_implicit_target_context_reset_between_iterations(self):
        """Context from log_response() in one iteration doesn't pollute the next."""
        from langwatch.experiment.experiment import (
            Experiment,
            _iteration_context,
            _target_context,
            IterationContext,
        )

        ev = Experiment("test-context-reset")
        ev.initialized = True
        ev.last_sent = 9999999999  # Prevent sending

        # First iteration with log_response (creates implicit Output target)
        iter_ctx1 = IterationContext(index=0, item={"question": "q1"})
        with ev._execute_item_iteration(0, {"question": "q1"}, in_thread=False):
            ev.log_response("response 1")
            ev.log("metric", index=0, score=0.5)

        # Context should be reset after iteration
        assert _target_context.get() is None, "Target context should be reset after iteration"

        # Second iteration WITHOUT log_response - should not inherit Output target
        with ev._execute_item_iteration(1, {"question": "q2"}, in_thread=False):
            # log() without log_response should NOT have a target
            ev.log("metric", index=1, score=0.6)

        # Check the evaluations
        evals = ev.batch["evaluations"]
        assert len(evals) == 2
        assert evals[0].target_id == "Output"  # First iteration used implicit target
        assert evals[1].target_id is None  # Second iteration should have no target

    def test_multiple_iterations_with_log_response_each_get_own_entry(self):
        """Each iteration with log_response creates its own dataset entry."""
        from langwatch.experiment.experiment import Experiment, _target_context

        ev = Experiment("test-multi-iterations")
        ev.initialized = True
        ev.last_sent = 9999999999

        # Run 3 iterations, each with log_response
        for i in range(3):
            with ev._execute_item_iteration(i, {"question": f"q{i}"}, in_thread=False):
                ev.log_response(f"response {i}")
                ev.log("metric", index=i, score=0.5 + i * 0.1)

        # Should have 3 dataset entries
        assert len(ev.batch["dataset"]) == 3
        for i, entry in enumerate(ev.batch["dataset"]):
            assert entry.target_id == "Output"
            assert entry.predicted == {"output": f"response {i}"}
            assert entry.index == i

        # Should have 3 evaluations, all with Output target
        assert len(ev.batch["evaluations"]) == 3
        for i, eval_result in enumerate(ev.batch["evaluations"]):
            assert eval_result.target_id == "Output"
            assert eval_result.index == i

    def test_run_without_log_response_after_run_with_log_response(self):
        """A run without log_response after a run with log_response should work correctly."""
        from langwatch.experiment.experiment import Experiment, _target_context

        # Simulate first run with log_response
        ev1 = Experiment("test-first-run")
        ev1.initialized = True
        ev1.last_sent = 9999999999

        for i in range(2):
            with ev1._execute_item_iteration(i, {"q": f"q{i}"}, in_thread=False):
                ev1.log_response(f"response {i}")
                ev1.log("metric", index=i, score=0.5)

        # Context should be clean after first experiment
        assert _target_context.get() is None

        # Simulate second run WITHOUT log_response (like the user's second scenario)
        ev2 = Experiment("test-second-run")
        ev2.initialized = True
        ev2.last_sent = 9999999999

        for i in range(2):
            with ev2._execute_item_iteration(i, {"q": f"q{i}"}, in_thread=False):
                # Just log, no log_response
                ev2.log("metric", index=i, score=0.6)

        # Second run evaluations should NOT have Output target
        for eval_result in ev2.batch["evaluations"]:
            assert eval_result.target_id is None, "Second run should not inherit Output target"



from enum import Enum


class PostApiAnalyticsTimeseriesBodyGroupBy(str, Enum):
    ERROR_HAS_ERROR = "error.has_error"
    EVALUATIONS_EVALUATION_LABEL = "evaluations.evaluation_label"
    EVALUATIONS_EVALUATION_PASSED = "evaluations.evaluation_passed"
    EVALUATIONS_EVALUATION_PROCESSING_STATE = "evaluations.evaluation_processing_state"
    EVENTS_EVENT_TYPE = "events.event_type"
    METADATA_CUSTOMER_ID = "metadata.customer_id"
    METADATA_LABELS = "metadata.labels"
    METADATA_MODEL = "metadata.model"
    METADATA_SPAN_TYPE = "metadata.span_type"
    METADATA_THREAD_ID = "metadata.thread_id"
    METADATA_USER_ID = "metadata.user_id"
    SENTIMENT_INPUT_SENTIMENT = "sentiment.input_sentiment"
    SENTIMENT_THUMBS_UP_DOWN = "sentiment.thumbs_up_down"
    TOPICS_TOPICS = "topics.topics"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.post_api_analytics_timeseries_body_series_item_pipeline_aggregation import (
    PostApiAnalyticsTimeseriesBodySeriesItemPipelineAggregation,
)
from ..models.post_api_analytics_timeseries_body_series_item_pipeline_field import (
    PostApiAnalyticsTimeseriesBodySeriesItemPipelineField,
)

T = TypeVar("T", bound="PostApiAnalyticsTimeseriesBodySeriesItemPipeline")


@_attrs_define
class PostApiAnalyticsTimeseriesBodySeriesItemPipeline:
    """
    Attributes:
        field (PostApiAnalyticsTimeseriesBodySeriesItemPipelineField):
        aggregation (PostApiAnalyticsTimeseriesBodySeriesItemPipelineAggregation):
    """

    field: PostApiAnalyticsTimeseriesBodySeriesItemPipelineField
    aggregation: PostApiAnalyticsTimeseriesBodySeriesItemPipelineAggregation
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field = self.field.value

        aggregation = self.aggregation.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "field": field,
                "aggregation": aggregation,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        field = PostApiAnalyticsTimeseriesBodySeriesItemPipelineField(d.pop("field"))

        aggregation = PostApiAnalyticsTimeseriesBodySeriesItemPipelineAggregation(d.pop("aggregation"))

        post_api_analytics_timeseries_body_series_item_pipeline = cls(
            field=field,
            aggregation=aggregation,
        )

        post_api_analytics_timeseries_body_series_item_pipeline.additional_properties = d
        return post_api_analytics_timeseries_body_series_item_pipeline

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.post_api_traces_search_response_200_pagination import PostApiTracesSearchResponse200Pagination


T = TypeVar("T", bound="PostApiTracesSearchResponse200")


@_attrs_define
class PostApiTracesSearchResponse200:
    """
    Attributes:
        traces (list[Any]):
        pagination (PostApiTracesSearchResponse200Pagination):
    """

    traces: list[Any]
    pagination: PostApiTracesSearchResponse200Pagination
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        traces = self.traces

        pagination = self.pagination.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "traces": traces,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_traces_search_response_200_pagination import PostApiTracesSearchResponse200Pagination

        d = dict(src_dict)
        traces = cast(list[Any], d.pop("traces"))

        pagination = PostApiTracesSearchResponse200Pagination.from_dict(d.pop("pagination"))

        post_api_traces_search_response_200 = cls(
            traces=traces,
            pagination=pagination,
        )

        post_api_traces_search_response_200.additional_properties = d
        return post_api_traces_search_response_200

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.post_api_traces_search_body_filters_annotations_has_annotation_type_1 import (
        PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType1,
    )
    from ..models.post_api_traces_search_body_filters_annotations_has_annotation_type_2 import (
        PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType2,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
        PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_guardrails_only_type_2 import (
        PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_type_1 import (
        PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType1,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_type_2 import (
        PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType2,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_label_type_1 import (
        PostApiTracesSearchBodyFiltersEvaluationsLabelType1,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_label_type_2 import (
        PostApiTracesSearchBodyFiltersEvaluationsLabelType2,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_passed_type_1 import (
        PostApiTracesSearchBodyFiltersEvaluationsPassedType1,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_passed_type_2 import (
        PostApiTracesSearchBodyFiltersEvaluationsPassedType2,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_score_type_1 import (
        PostApiTracesSearchBodyFiltersEvaluationsScoreType1,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_score_type_2 import (
        PostApiTracesSearchBodyFiltersEvaluationsScoreType2,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_state_type_1 import (
        PostApiTracesSearchBodyFiltersEvaluationsStateType1,
    )
    from ..models.post_api_traces_search_body_filters_evaluations_state_type_2 import (
        PostApiTracesSearchBodyFiltersEvaluationsStateType2,
    )
    from ..models.post_api_traces_search_body_filters_events_event_details_key_type_1 import (
        PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType1,
    )
    from ..models.post_api_traces_search_body_filters_events_event_details_key_type_2 import (
        PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType2,
    )
    from ..models.post_api_traces_search_body_filters_events_event_type_type_1 import (
        PostApiTracesSearchBodyFiltersEventsEventTypeType1,
    )
    from ..models.post_api_traces_search_body_filters_events_event_type_type_2 import (
        PostApiTracesSearchBodyFiltersEventsEventTypeType2,
    )
    from ..models.post_api_traces_search_body_filters_events_metrics_key_type_1 import (
        PostApiTracesSearchBodyFiltersEventsMetricsKeyType1,
    )
    from ..models.post_api_traces_search_body_filters_events_metrics_key_type_2 import (
        PostApiTracesSearchBodyFiltersEventsMetricsKeyType2,
    )
    from ..models.post_api_traces_search_body_filters_events_metrics_value_type_1 import (
        PostApiTracesSearchBodyFiltersEventsMetricsValueType1,
    )
    from ..models.post_api_traces_search_body_filters_events_metrics_value_type_2 import (
        PostApiTracesSearchBodyFiltersEventsMetricsValueType2,
    )
    from ..models.post_api_traces_search_body_filters_metadata_customer_id_type_1 import (
        PostApiTracesSearchBodyFiltersMetadataCustomerIdType1,
    )
    from ..models.post_api_traces_search_body_filters_metadata_customer_id_type_2 import (
        PostApiTracesSearchBodyFiltersMetadataCustomerIdType2,
    )
    from ..models.post_api_traces_search_body_filters_metadata_key_type_1 import (
        PostApiTracesSearchBodyFiltersMetadataKeyType1,
    )
    from ..models.post_api_traces_search_body_filters_metadata_key_type_2 import (
        PostApiTracesSearchBodyFiltersMetadataKeyType2,
    )
    from ..models.post_api_traces_search_body_filters_metadata_labels_type_1 import (
        PostApiTracesSearchBodyFiltersMetadataLabelsType1,
    )
    from ..models.post_api_traces_search_body_filters_metadata_labels_type_2 import (
        PostApiTracesSearchBodyFiltersMetadataLabelsType2,
    )
    from ..models.post_api_traces_search_body_filters_metadata_prompt_ids_type_1 import (
        PostApiTracesSearchBodyFiltersMetadataPromptIdsType1,
    )
    from ..models.post_api_traces_search_body_filters_metadata_prompt_ids_type_2 import (
        PostApiTracesSearchBodyFiltersMetadataPromptIdsType2,
    )
    from ..models.post_api_traces_search_body_filters_metadata_thread_id_type_1 import (
        PostApiTracesSearchBodyFiltersMetadataThreadIdType1,
    )
    from ..models.post_api_traces_search_body_filters_metadata_thread_id_type_2 import (
        PostApiTracesSearchBodyFiltersMetadataThreadIdType2,
    )
    from ..models.post_api_traces_search_body_filters_metadata_user_id_type_1 import (
        PostApiTracesSearchBodyFiltersMetadataUserIdType1,
    )
    from ..models.post_api_traces_search_body_filters_metadata_user_id_type_2 import (
        PostApiTracesSearchBodyFiltersMetadataUserIdType2,
    )
    from ..models.post_api_traces_search_body_filters_metadata_value_type_1 import (
        PostApiTracesSearchBodyFiltersMetadataValueType1,
    )
    from ..models.post_api_traces_search_body_filters_metadata_value_type_2 import (
        PostApiTracesSearchBodyFiltersMetadataValueType2,
    )
    from ..models.post_api_traces_search_body_filters_sentiment_input_sentiment_type_1 import (
        PostApiTracesSearchBodyFiltersSentimentInputSentimentType1,
    )
    from ..models.post_api_traces_search_body_filters_sentiment_input_sentiment_type_2 import (
        PostApiTracesSearchBodyFiltersSentimentInputSentimentType2,
    )
    from ..models.post_api_traces_search_body_filters_spans_model_type_1 import (
        PostApiTracesSearchBodyFiltersSpansModelType1,
    )
    from ..models.post_api_traces_search_body_filters_spans_model_type_2 import (
        PostApiTracesSearchBodyFiltersSpansModelType2,
    )
    from ..models.post_api_traces_search_body_filters_spans_type_type_1 import (
        PostApiTracesSearchBodyFiltersSpansTypeType1,
    )
    from ..models.post_api_traces_search_body_filters_spans_type_type_2 import (
        PostApiTracesSearchBodyFiltersSpansTypeType2,
    )
    from ..models.post_api_traces_search_body_filters_topics_subtopics_type_1 import (
        PostApiTracesSearchBodyFiltersTopicsSubtopicsType1,
    )
    from ..models.post_api_traces_search_body_filters_topics_subtopics_type_2 import (
        PostApiTracesSearchBodyFiltersTopicsSubtopicsType2,
    )
    from ..models.post_api_traces_search_body_filters_topics_topics_type_1 import (
        PostApiTracesSearchBodyFiltersTopicsTopicsType1,
    )
    from ..models.post_api_traces_search_body_filters_topics_topics_type_2 import (
        PostApiTracesSearchBodyFiltersTopicsTopicsType2,
    )
    from ..models.post_api_traces_search_body_filters_traces_error_type_1 import (
        PostApiTracesSearchBodyFiltersTracesErrorType1,
    )
    from ..models.post_api_traces_search_body_filters_traces_error_type_2 import (
        PostApiTracesSearchBodyFiltersTracesErrorType2,
    )


T = TypeVar("T", bound="PostApiTracesSearchBodyFilters")


@_attrs_define
class PostApiTracesSearchBodyFilters:
    """
    Attributes:
        topics_topics (list[str] | PostApiTracesSearchBodyFiltersTopicsTopicsType1 |
            PostApiTracesSearchBodyFiltersTopicsTopicsType2 | Unset):
        topics_subtopics (list[str] | PostApiTracesSearchBodyFiltersTopicsSubtopicsType1 |
            PostApiTracesSearchBodyFiltersTopicsSubtopicsType2 | Unset):
        metadata_user_id (list[str] | PostApiTracesSearchBodyFiltersMetadataUserIdType1 |
            PostApiTracesSearchBodyFiltersMetadataUserIdType2 | Unset):
        metadata_thread_id (list[str] | PostApiTracesSearchBodyFiltersMetadataThreadIdType1 |
            PostApiTracesSearchBodyFiltersMetadataThreadIdType2 | Unset):
        metadata_customer_id (list[str] | PostApiTracesSearchBodyFiltersMetadataCustomerIdType1 |
            PostApiTracesSearchBodyFiltersMetadataCustomerIdType2 | Unset):
        metadata_labels (list[str] | PostApiTracesSearchBodyFiltersMetadataLabelsType1 |
            PostApiTracesSearchBodyFiltersMetadataLabelsType2 | Unset):
        metadata_key (list[str] | PostApiTracesSearchBodyFiltersMetadataKeyType1 |
            PostApiTracesSearchBodyFiltersMetadataKeyType2 | Unset):
        metadata_value (list[str] | PostApiTracesSearchBodyFiltersMetadataValueType1 |
            PostApiTracesSearchBodyFiltersMetadataValueType2 | Unset):
        metadata_prompt_ids (list[str] | PostApiTracesSearchBodyFiltersMetadataPromptIdsType1 |
            PostApiTracesSearchBodyFiltersMetadataPromptIdsType2 | Unset):
        traces_error (list[str] | PostApiTracesSearchBodyFiltersTracesErrorType1 |
            PostApiTracesSearchBodyFiltersTracesErrorType2 | Unset):
        spans_type (list[str] | PostApiTracesSearchBodyFiltersSpansTypeType1 |
            PostApiTracesSearchBodyFiltersSpansTypeType2 | Unset):
        spans_model (list[str] | PostApiTracesSearchBodyFiltersSpansModelType1 |
            PostApiTracesSearchBodyFiltersSpansModelType2 | Unset):
        evaluations_evaluator_id (list[str] | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType1 |
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType2 | Unset):
        evaluations_evaluator_id_guardrails_only (list[str] |
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1 |
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2 | Unset):
        evaluations_passed (list[str] | PostApiTracesSearchBodyFiltersEvaluationsPassedType1 |
            PostApiTracesSearchBodyFiltersEvaluationsPassedType2 | Unset):
        evaluations_score (list[str] | PostApiTracesSearchBodyFiltersEvaluationsScoreType1 |
            PostApiTracesSearchBodyFiltersEvaluationsScoreType2 | Unset):
        evaluations_state (list[str] | PostApiTracesSearchBodyFiltersEvaluationsStateType1 |
            PostApiTracesSearchBodyFiltersEvaluationsStateType2 | Unset):
        evaluations_label (list[str] | PostApiTracesSearchBodyFiltersEvaluationsLabelType1 |
            PostApiTracesSearchBodyFiltersEvaluationsLabelType2 | Unset):
        events_event_type (list[str] | PostApiTracesSearchBodyFiltersEventsEventTypeType1 |
            PostApiTracesSearchBodyFiltersEventsEventTypeType2 | Unset):
        events_metrics_key (list[str] | PostApiTracesSearchBodyFiltersEventsMetricsKeyType1 |
            PostApiTracesSearchBodyFiltersEventsMetricsKeyType2 | Unset):
        events_metrics_value (list[str] | PostApiTracesSearchBodyFiltersEventsMetricsValueType1 |
            PostApiTracesSearchBodyFiltersEventsMetricsValueType2 | Unset):
        events_event_details_key (list[str] | PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType1 |
            PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType2 | Unset):
        annotations_has_annotation (list[str] | PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType1 |
            PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType2 | Unset):
        sentiment_input_sentiment (list[str] | PostApiTracesSearchBodyFiltersSentimentInputSentimentType1 |
            PostApiTracesSearchBodyFiltersSentimentInputSentimentType2 | Unset):
    """

    topics_topics: (
        list[str]
        | PostApiTracesSearchBodyFiltersTopicsTopicsType1
        | PostApiTracesSearchBodyFiltersTopicsTopicsType2
        | Unset
    ) = UNSET
    topics_subtopics: (
        list[str]
        | PostApiTracesSearchBodyFiltersTopicsSubtopicsType1
        | PostApiTracesSearchBodyFiltersTopicsSubtopicsType2
        | Unset
    ) = UNSET
    metadata_user_id: (
        list[str]
        | PostApiTracesSearchBodyFiltersMetadataUserIdType1
        | PostApiTracesSearchBodyFiltersMetadataUserIdType2
        | Unset
    ) = UNSET
    metadata_thread_id: (
        list[str]
        | PostApiTracesSearchBodyFiltersMetadataThreadIdType1
        | PostApiTracesSearchBodyFiltersMetadataThreadIdType2
        | Unset
    ) = UNSET
    metadata_customer_id: (
        list[str]
        | PostApiTracesSearchBodyFiltersMetadataCustomerIdType1
        | PostApiTracesSearchBodyFiltersMetadataCustomerIdType2
        | Unset
    ) = UNSET
    metadata_labels: (
        list[str]
        | PostApiTracesSearchBodyFiltersMetadataLabelsType1
        | PostApiTracesSearchBodyFiltersMetadataLabelsType2
        | Unset
    ) = UNSET
    metadata_key: (
        list[str]
        | PostApiTracesSearchBodyFiltersMetadataKeyType1
        | PostApiTracesSearchBodyFiltersMetadataKeyType2
        | Unset
    ) = UNSET
    metadata_value: (
        list[str]
        | PostApiTracesSearchBodyFiltersMetadataValueType1
        | PostApiTracesSearchBodyFiltersMetadataValueType2
        | Unset
    ) = UNSET
    metadata_prompt_ids: (
        list[str]
        | PostApiTracesSearchBodyFiltersMetadataPromptIdsType1
        | PostApiTracesSearchBodyFiltersMetadataPromptIdsType2
        | Unset
    ) = UNSET
    traces_error: (
        list[str]
        | PostApiTracesSearchBodyFiltersTracesErrorType1
        | PostApiTracesSearchBodyFiltersTracesErrorType2
        | Unset
    ) = UNSET
    spans_type: (
        list[str] | PostApiTracesSearchBodyFiltersSpansTypeType1 | PostApiTracesSearchBodyFiltersSpansTypeType2 | Unset
    ) = UNSET
    spans_model: (
        list[str]
        | PostApiTracesSearchBodyFiltersSpansModelType1
        | PostApiTracesSearchBodyFiltersSpansModelType2
        | Unset
    ) = UNSET
    evaluations_evaluator_id: (
        list[str]
        | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType1
        | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType2
        | Unset
    ) = UNSET
    evaluations_evaluator_id_guardrails_only: (
        list[str]
        | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1
        | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2
        | Unset
    ) = UNSET
    evaluations_passed: (
        list[str]
        | PostApiTracesSearchBodyFiltersEvaluationsPassedType1
        | PostApiTracesSearchBodyFiltersEvaluationsPassedType2
        | Unset
    ) = UNSET
    evaluations_score: (
        list[str]
        | PostApiTracesSearchBodyFiltersEvaluationsScoreType1
        | PostApiTracesSearchBodyFiltersEvaluationsScoreType2
        | Unset
    ) = UNSET
    evaluations_state: (
        list[str]
        | PostApiTracesSearchBodyFiltersEvaluationsStateType1
        | PostApiTracesSearchBodyFiltersEvaluationsStateType2
        | Unset
    ) = UNSET
    evaluations_label: (
        list[str]
        | PostApiTracesSearchBodyFiltersEvaluationsLabelType1
        | PostApiTracesSearchBodyFiltersEvaluationsLabelType2
        | Unset
    ) = UNSET
    events_event_type: (
        list[str]
        | PostApiTracesSearchBodyFiltersEventsEventTypeType1
        | PostApiTracesSearchBodyFiltersEventsEventTypeType2
        | Unset
    ) = UNSET
    events_metrics_key: (
        list[str]
        | PostApiTracesSearchBodyFiltersEventsMetricsKeyType1
        | PostApiTracesSearchBodyFiltersEventsMetricsKeyType2
        | Unset
    ) = UNSET
    events_metrics_value: (
        list[str]
        | PostApiTracesSearchBodyFiltersEventsMetricsValueType1
        | PostApiTracesSearchBodyFiltersEventsMetricsValueType2
        | Unset
    ) = UNSET
    events_event_details_key: (
        list[str]
        | PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType1
        | PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType2
        | Unset
    ) = UNSET
    annotations_has_annotation: (
        list[str]
        | PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType1
        | PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType2
        | Unset
    ) = UNSET
    sentiment_input_sentiment: (
        list[str]
        | PostApiTracesSearchBodyFiltersSentimentInputSentimentType1
        | PostApiTracesSearchBodyFiltersSentimentInputSentimentType2
        | Unset
    ) = UNSET

    def to_dict(self) -> dict[str, Any]:
        from ..models.post_api_traces_search_body_filters_annotations_has_annotation_type_1 import (
            PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_label_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsLabelType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_passed_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsPassedType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_score_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsScoreType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_state_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsStateType1,
        )
        from ..models.post_api_traces_search_body_filters_events_event_details_key_type_1 import (
            PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType1,
        )
        from ..models.post_api_traces_search_body_filters_events_event_type_type_1 import (
            PostApiTracesSearchBodyFiltersEventsEventTypeType1,
        )
        from ..models.post_api_traces_search_body_filters_events_metrics_key_type_1 import (
            PostApiTracesSearchBodyFiltersEventsMetricsKeyType1,
        )
        from ..models.post_api_traces_search_body_filters_events_metrics_value_type_1 import (
            PostApiTracesSearchBodyFiltersEventsMetricsValueType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_customer_id_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataCustomerIdType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_key_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataKeyType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_labels_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataLabelsType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_prompt_ids_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataPromptIdsType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_thread_id_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataThreadIdType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_user_id_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataUserIdType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_value_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataValueType1,
        )
        from ..models.post_api_traces_search_body_filters_sentiment_input_sentiment_type_1 import (
            PostApiTracesSearchBodyFiltersSentimentInputSentimentType1,
        )
        from ..models.post_api_traces_search_body_filters_spans_model_type_1 import (
            PostApiTracesSearchBodyFiltersSpansModelType1,
        )
        from ..models.post_api_traces_search_body_filters_spans_type_type_1 import (
            PostApiTracesSearchBodyFiltersSpansTypeType1,
        )
        from ..models.post_api_traces_search_body_filters_topics_subtopics_type_1 import (
            PostApiTracesSearchBodyFiltersTopicsSubtopicsType1,
        )
        from ..models.post_api_traces_search_body_filters_topics_topics_type_1 import (
            PostApiTracesSearchBodyFiltersTopicsTopicsType1,
        )
        from ..models.post_api_traces_search_body_filters_traces_error_type_1 import (
            PostApiTracesSearchBodyFiltersTracesErrorType1,
        )

        topics_topics: dict[str, Any] | list[str] | Unset
        if isinstance(self.topics_topics, Unset):
            topics_topics = UNSET
        elif isinstance(self.topics_topics, list):
            topics_topics = self.topics_topics

        elif isinstance(self.topics_topics, PostApiTracesSearchBodyFiltersTopicsTopicsType1):
            topics_topics = self.topics_topics.to_dict()
        else:
            topics_topics = self.topics_topics.to_dict()

        topics_subtopics: dict[str, Any] | list[str] | Unset
        if isinstance(self.topics_subtopics, Unset):
            topics_subtopics = UNSET
        elif isinstance(self.topics_subtopics, list):
            topics_subtopics = self.topics_subtopics

        elif isinstance(self.topics_subtopics, PostApiTracesSearchBodyFiltersTopicsSubtopicsType1):
            topics_subtopics = self.topics_subtopics.to_dict()
        else:
            topics_subtopics = self.topics_subtopics.to_dict()

        metadata_user_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_user_id, Unset):
            metadata_user_id = UNSET
        elif isinstance(self.metadata_user_id, list):
            metadata_user_id = self.metadata_user_id

        elif isinstance(self.metadata_user_id, PostApiTracesSearchBodyFiltersMetadataUserIdType1):
            metadata_user_id = self.metadata_user_id.to_dict()
        else:
            metadata_user_id = self.metadata_user_id.to_dict()

        metadata_thread_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_thread_id, Unset):
            metadata_thread_id = UNSET
        elif isinstance(self.metadata_thread_id, list):
            metadata_thread_id = self.metadata_thread_id

        elif isinstance(self.metadata_thread_id, PostApiTracesSearchBodyFiltersMetadataThreadIdType1):
            metadata_thread_id = self.metadata_thread_id.to_dict()
        else:
            metadata_thread_id = self.metadata_thread_id.to_dict()

        metadata_customer_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_customer_id, Unset):
            metadata_customer_id = UNSET
        elif isinstance(self.metadata_customer_id, list):
            metadata_customer_id = self.metadata_customer_id

        elif isinstance(self.metadata_customer_id, PostApiTracesSearchBodyFiltersMetadataCustomerIdType1):
            metadata_customer_id = self.metadata_customer_id.to_dict()
        else:
            metadata_customer_id = self.metadata_customer_id.to_dict()

        metadata_labels: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_labels, Unset):
            metadata_labels = UNSET
        elif isinstance(self.metadata_labels, list):
            metadata_labels = self.metadata_labels

        elif isinstance(self.metadata_labels, PostApiTracesSearchBodyFiltersMetadataLabelsType1):
            metadata_labels = self.metadata_labels.to_dict()
        else:
            metadata_labels = self.metadata_labels.to_dict()

        metadata_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_key, Unset):
            metadata_key = UNSET
        elif isinstance(self.metadata_key, list):
            metadata_key = self.metadata_key

        elif isinstance(self.metadata_key, PostApiTracesSearchBodyFiltersMetadataKeyType1):
            metadata_key = self.metadata_key.to_dict()
        else:
            metadata_key = self.metadata_key.to_dict()

        metadata_value: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_value, Unset):
            metadata_value = UNSET
        elif isinstance(self.metadata_value, list):
            metadata_value = self.metadata_value

        elif isinstance(self.metadata_value, PostApiTracesSearchBodyFiltersMetadataValueType1):
            metadata_value = self.metadata_value.to_dict()
        else:
            metadata_value = self.metadata_value.to_dict()

        metadata_prompt_ids: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_prompt_ids, Unset):
            metadata_prompt_ids = UNSET
        elif isinstance(self.metadata_prompt_ids, list):
            metadata_prompt_ids = self.metadata_prompt_ids

        elif isinstance(self.metadata_prompt_ids, PostApiTracesSearchBodyFiltersMetadataPromptIdsType1):
            metadata_prompt_ids = self.metadata_prompt_ids.to_dict()
        else:
            metadata_prompt_ids = self.metadata_prompt_ids.to_dict()

        traces_error: dict[str, Any] | list[str] | Unset
        if isinstance(self.traces_error, Unset):
            traces_error = UNSET
        elif isinstance(self.traces_error, list):
            traces_error = self.traces_error

        elif isinstance(self.traces_error, PostApiTracesSearchBodyFiltersTracesErrorType1):
            traces_error = self.traces_error.to_dict()
        else:
            traces_error = self.traces_error.to_dict()

        spans_type: dict[str, Any] | list[str] | Unset
        if isinstance(self.spans_type, Unset):
            spans_type = UNSET
        elif isinstance(self.spans_type, list):
            spans_type = self.spans_type

        elif isinstance(self.spans_type, PostApiTracesSearchBodyFiltersSpansTypeType1):
            spans_type = self.spans_type.to_dict()
        else:
            spans_type = self.spans_type.to_dict()

        spans_model: dict[str, Any] | list[str] | Unset
        if isinstance(self.spans_model, Unset):
            spans_model = UNSET
        elif isinstance(self.spans_model, list):
            spans_model = self.spans_model

        elif isinstance(self.spans_model, PostApiTracesSearchBodyFiltersSpansModelType1):
            spans_model = self.spans_model.to_dict()
        else:
            spans_model = self.spans_model.to_dict()

        evaluations_evaluator_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_evaluator_id, Unset):
            evaluations_evaluator_id = UNSET
        elif isinstance(self.evaluations_evaluator_id, list):
            evaluations_evaluator_id = self.evaluations_evaluator_id

        elif isinstance(self.evaluations_evaluator_id, PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType1):
            evaluations_evaluator_id = self.evaluations_evaluator_id.to_dict()
        else:
            evaluations_evaluator_id = self.evaluations_evaluator_id.to_dict()

        evaluations_evaluator_id_guardrails_only: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_evaluator_id_guardrails_only, Unset):
            evaluations_evaluator_id_guardrails_only = UNSET
        elif isinstance(self.evaluations_evaluator_id_guardrails_only, list):
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only

        elif isinstance(
            self.evaluations_evaluator_id_guardrails_only,
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        ):
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only.to_dict()
        else:
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only.to_dict()

        evaluations_passed: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_passed, Unset):
            evaluations_passed = UNSET
        elif isinstance(self.evaluations_passed, list):
            evaluations_passed = self.evaluations_passed

        elif isinstance(self.evaluations_passed, PostApiTracesSearchBodyFiltersEvaluationsPassedType1):
            evaluations_passed = self.evaluations_passed.to_dict()
        else:
            evaluations_passed = self.evaluations_passed.to_dict()

        evaluations_score: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_score, Unset):
            evaluations_score = UNSET
        elif isinstance(self.evaluations_score, list):
            evaluations_score = self.evaluations_score

        elif isinstance(self.evaluations_score, PostApiTracesSearchBodyFiltersEvaluationsScoreType1):
            evaluations_score = self.evaluations_score.to_dict()
        else:
            evaluations_score = self.evaluations_score.to_dict()

        evaluations_state: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_state, Unset):
            evaluations_state = UNSET
        elif isinstance(self.evaluations_state, list):
            evaluations_state = self.evaluations_state

        elif isinstance(self.evaluations_state, PostApiTracesSearchBodyFiltersEvaluationsStateType1):
            evaluations_state = self.evaluations_state.to_dict()
        else:
            evaluations_state = self.evaluations_state.to_dict()

        evaluations_label: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_label, Unset):
            evaluations_label = UNSET
        elif isinstance(self.evaluations_label, list):
            evaluations_label = self.evaluations_label

        elif isinstance(self.evaluations_label, PostApiTracesSearchBodyFiltersEvaluationsLabelType1):
            evaluations_label = self.evaluations_label.to_dict()
        else:
            evaluations_label = self.evaluations_label.to_dict()

        events_event_type: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_event_type, Unset):
            events_event_type = UNSET
        elif isinstance(self.events_event_type, list):
            events_event_type = self.events_event_type

        elif isinstance(self.events_event_type, PostApiTracesSearchBodyFiltersEventsEventTypeType1):
            events_event_type = self.events_event_type.to_dict()
        else:
            events_event_type = self.events_event_type.to_dict()

        events_metrics_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_metrics_key, Unset):
            events_metrics_key = UNSET
        elif isinstance(self.events_metrics_key, list):
            events_metrics_key = self.events_metrics_key

        elif isinstance(self.events_metrics_key, PostApiTracesSearchBodyFiltersEventsMetricsKeyType1):
            events_metrics_key = self.events_metrics_key.to_dict()
        else:
            events_metrics_key = self.events_metrics_key.to_dict()

        events_metrics_value: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_metrics_value, Unset):
            events_metrics_value = UNSET
        elif isinstance(self.events_metrics_value, list):
            events_metrics_value = self.events_metrics_value

        elif isinstance(self.events_metrics_value, PostApiTracesSearchBodyFiltersEventsMetricsValueType1):
            events_metrics_value = self.events_metrics_value.to_dict()
        else:
            events_metrics_value = self.events_metrics_value.to_dict()

        events_event_details_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_event_details_key, Unset):
            events_event_details_key = UNSET
        elif isinstance(self.events_event_details_key, list):
            events_event_details_key = self.events_event_details_key

        elif isinstance(self.events_event_details_key, PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType1):
            events_event_details_key = self.events_event_details_key.to_dict()
        else:
            events_event_details_key = self.events_event_details_key.to_dict()

        annotations_has_annotation: dict[str, Any] | list[str] | Unset
        if isinstance(self.annotations_has_annotation, Unset):
            annotations_has_annotation = UNSET
        elif isinstance(self.annotations_has_annotation, list):
            annotations_has_annotation = self.annotations_has_annotation

        elif isinstance(self.annotations_has_annotation, PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType1):
            annotations_has_annotation = self.annotations_has_annotation.to_dict()
        else:
            annotations_has_annotation = self.annotations_has_annotation.to_dict()

        sentiment_input_sentiment: dict[str, Any] | list[str] | Unset
        if isinstance(self.sentiment_input_sentiment, Unset):
            sentiment_input_sentiment = UNSET
        elif isinstance(self.sentiment_input_sentiment, list):
            sentiment_input_sentiment = self.sentiment_input_sentiment

        elif isinstance(self.sentiment_input_sentiment, PostApiTracesSearchBodyFiltersSentimentInputSentimentType1):
            sentiment_input_sentiment = self.sentiment_input_sentiment.to_dict()
        else:
            sentiment_input_sentiment = self.sentiment_input_sentiment.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if topics_topics is not UNSET:
            field_dict["topics.topics"] = topics_topics
        if topics_subtopics is not UNSET:
            field_dict["topics.subtopics"] = topics_subtopics
        if metadata_user_id is not UNSET:
            field_dict["metadata.user_id"] = metadata_user_id
        if metadata_thread_id is not UNSET:
            field_dict["metadata.thread_id"] = metadata_thread_id
        if metadata_customer_id is not UNSET:
            field_dict["metadata.customer_id"] = metadata_customer_id
        if metadata_labels is not UNSET:
            field_dict["metadata.labels"] = metadata_labels
        if metadata_key is not UNSET:
            field_dict["metadata.key"] = metadata_key
        if metadata_value is not UNSET:
            field_dict["metadata.value"] = metadata_value
        if metadata_prompt_ids is not UNSET:
            field_dict["metadata.prompt_ids"] = metadata_prompt_ids
        if traces_error is not UNSET:
            field_dict["traces.error"] = traces_error
        if spans_type is not UNSET:
            field_dict["spans.type"] = spans_type
        if spans_model is not UNSET:
            field_dict["spans.model"] = spans_model
        if evaluations_evaluator_id is not UNSET:
            field_dict["evaluations.evaluator_id"] = evaluations_evaluator_id
        if evaluations_evaluator_id_guardrails_only is not UNSET:
            field_dict["evaluations.evaluator_id.guardrails_only"] = evaluations_evaluator_id_guardrails_only
        if evaluations_passed is not UNSET:
            field_dict["evaluations.passed"] = evaluations_passed
        if evaluations_score is not UNSET:
            field_dict["evaluations.score"] = evaluations_score
        if evaluations_state is not UNSET:
            field_dict["evaluations.state"] = evaluations_state
        if evaluations_label is not UNSET:
            field_dict["evaluations.label"] = evaluations_label
        if events_event_type is not UNSET:
            field_dict["events.event_type"] = events_event_type
        if events_metrics_key is not UNSET:
            field_dict["events.metrics.key"] = events_metrics_key
        if events_metrics_value is not UNSET:
            field_dict["events.metrics.value"] = events_metrics_value
        if events_event_details_key is not UNSET:
            field_dict["events.event_details.key"] = events_event_details_key
        if annotations_has_annotation is not UNSET:
            field_dict["annotations.hasAnnotation"] = annotations_has_annotation
        if sentiment_input_sentiment is not UNSET:
            field_dict["sentiment.input_sentiment"] = sentiment_input_sentiment

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_traces_search_body_filters_annotations_has_annotation_type_1 import (
            PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType1,
        )
        from ..models.post_api_traces_search_body_filters_annotations_has_annotation_type_2 import (
            PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType2,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_guardrails_only_type_2 import (
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_evaluator_id_type_2 import (
            PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType2,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_label_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsLabelType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_label_type_2 import (
            PostApiTracesSearchBodyFiltersEvaluationsLabelType2,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_passed_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsPassedType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_passed_type_2 import (
            PostApiTracesSearchBodyFiltersEvaluationsPassedType2,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_score_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsScoreType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_score_type_2 import (
            PostApiTracesSearchBodyFiltersEvaluationsScoreType2,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_state_type_1 import (
            PostApiTracesSearchBodyFiltersEvaluationsStateType1,
        )
        from ..models.post_api_traces_search_body_filters_evaluations_state_type_2 import (
            PostApiTracesSearchBodyFiltersEvaluationsStateType2,
        )
        from ..models.post_api_traces_search_body_filters_events_event_details_key_type_1 import (
            PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType1,
        )
        from ..models.post_api_traces_search_body_filters_events_event_details_key_type_2 import (
            PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType2,
        )
        from ..models.post_api_traces_search_body_filters_events_event_type_type_1 import (
            PostApiTracesSearchBodyFiltersEventsEventTypeType1,
        )
        from ..models.post_api_traces_search_body_filters_events_event_type_type_2 import (
            PostApiTracesSearchBodyFiltersEventsEventTypeType2,
        )
        from ..models.post_api_traces_search_body_filters_events_metrics_key_type_1 import (
            PostApiTracesSearchBodyFiltersEventsMetricsKeyType1,
        )
        from ..models.post_api_traces_search_body_filters_events_metrics_key_type_2 import (
            PostApiTracesSearchBodyFiltersEventsMetricsKeyType2,
        )
        from ..models.post_api_traces_search_body_filters_events_metrics_value_type_1 import (
            PostApiTracesSearchBodyFiltersEventsMetricsValueType1,
        )
        from ..models.post_api_traces_search_body_filters_events_metrics_value_type_2 import (
            PostApiTracesSearchBodyFiltersEventsMetricsValueType2,
        )
        from ..models.post_api_traces_search_body_filters_metadata_customer_id_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataCustomerIdType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_customer_id_type_2 import (
            PostApiTracesSearchBodyFiltersMetadataCustomerIdType2,
        )
        from ..models.post_api_traces_search_body_filters_metadata_key_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataKeyType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_key_type_2 import (
            PostApiTracesSearchBodyFiltersMetadataKeyType2,
        )
        from ..models.post_api_traces_search_body_filters_metadata_labels_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataLabelsType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_labels_type_2 import (
            PostApiTracesSearchBodyFiltersMetadataLabelsType2,
        )
        from ..models.post_api_traces_search_body_filters_metadata_prompt_ids_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataPromptIdsType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_prompt_ids_type_2 import (
            PostApiTracesSearchBodyFiltersMetadataPromptIdsType2,
        )
        from ..models.post_api_traces_search_body_filters_metadata_thread_id_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataThreadIdType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_thread_id_type_2 import (
            PostApiTracesSearchBodyFiltersMetadataThreadIdType2,
        )
        from ..models.post_api_traces_search_body_filters_metadata_user_id_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataUserIdType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_user_id_type_2 import (
            PostApiTracesSearchBodyFiltersMetadataUserIdType2,
        )
        from ..models.post_api_traces_search_body_filters_metadata_value_type_1 import (
            PostApiTracesSearchBodyFiltersMetadataValueType1,
        )
        from ..models.post_api_traces_search_body_filters_metadata_value_type_2 import (
            PostApiTracesSearchBodyFiltersMetadataValueType2,
        )
        from ..models.post_api_traces_search_body_filters_sentiment_input_sentiment_type_1 import (
            PostApiTracesSearchBodyFiltersSentimentInputSentimentType1,
        )
        from ..models.post_api_traces_search_body_filters_sentiment_input_sentiment_type_2 import (
            PostApiTracesSearchBodyFiltersSentimentInputSentimentType2,
        )
        from ..models.post_api_traces_search_body_filters_spans_model_type_1 import (
            PostApiTracesSearchBodyFiltersSpansModelType1,
        )
        from ..models.post_api_traces_search_body_filters_spans_model_type_2 import (
            PostApiTracesSearchBodyFiltersSpansModelType2,
        )
        from ..models.post_api_traces_search_body_filters_spans_type_type_1 import (
            PostApiTracesSearchBodyFiltersSpansTypeType1,
        )
        from ..models.post_api_traces_search_body_filters_spans_type_type_2 import (
            PostApiTracesSearchBodyFiltersSpansTypeType2,
        )
        from ..models.post_api_traces_search_body_filters_topics_subtopics_type_1 import (
            PostApiTracesSearchBodyFiltersTopicsSubtopicsType1,
        )
        from ..models.post_api_traces_search_body_filters_topics_subtopics_type_2 import (
            PostApiTracesSearchBodyFiltersTopicsSubtopicsType2,
        )
        from ..models.post_api_traces_search_body_filters_topics_topics_type_1 import (
            PostApiTracesSearchBodyFiltersTopicsTopicsType1,
        )
        from ..models.post_api_traces_search_body_filters_topics_topics_type_2 import (
            PostApiTracesSearchBodyFiltersTopicsTopicsType2,
        )
        from ..models.post_api_traces_search_body_filters_traces_error_type_1 import (
            PostApiTracesSearchBodyFiltersTracesErrorType1,
        )
        from ..models.post_api_traces_search_body_filters_traces_error_type_2 import (
            PostApiTracesSearchBodyFiltersTracesErrorType2,
        )

        d = dict(src_dict)

        def _parse_topics_topics(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersTopicsTopicsType1
            | PostApiTracesSearchBodyFiltersTopicsTopicsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                topics_topics_type_0 = cast(list[str], data)

                return topics_topics_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                topics_topics_type_1 = PostApiTracesSearchBodyFiltersTopicsTopicsType1.from_dict(data)

                return topics_topics_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            topics_topics_type_2 = PostApiTracesSearchBodyFiltersTopicsTopicsType2.from_dict(data)

            return topics_topics_type_2

        topics_topics = _parse_topics_topics(d.pop("topics.topics", UNSET))

        def _parse_topics_subtopics(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersTopicsSubtopicsType1
            | PostApiTracesSearchBodyFiltersTopicsSubtopicsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                topics_subtopics_type_0 = cast(list[str], data)

                return topics_subtopics_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                topics_subtopics_type_1 = PostApiTracesSearchBodyFiltersTopicsSubtopicsType1.from_dict(data)

                return topics_subtopics_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            topics_subtopics_type_2 = PostApiTracesSearchBodyFiltersTopicsSubtopicsType2.from_dict(data)

            return topics_subtopics_type_2

        topics_subtopics = _parse_topics_subtopics(d.pop("topics.subtopics", UNSET))

        def _parse_metadata_user_id(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersMetadataUserIdType1
            | PostApiTracesSearchBodyFiltersMetadataUserIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_user_id_type_0 = cast(list[str], data)

                return metadata_user_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_user_id_type_1 = PostApiTracesSearchBodyFiltersMetadataUserIdType1.from_dict(data)

                return metadata_user_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_user_id_type_2 = PostApiTracesSearchBodyFiltersMetadataUserIdType2.from_dict(data)

            return metadata_user_id_type_2

        metadata_user_id = _parse_metadata_user_id(d.pop("metadata.user_id", UNSET))

        def _parse_metadata_thread_id(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersMetadataThreadIdType1
            | PostApiTracesSearchBodyFiltersMetadataThreadIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_thread_id_type_0 = cast(list[str], data)

                return metadata_thread_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_thread_id_type_1 = PostApiTracesSearchBodyFiltersMetadataThreadIdType1.from_dict(data)

                return metadata_thread_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_thread_id_type_2 = PostApiTracesSearchBodyFiltersMetadataThreadIdType2.from_dict(data)

            return metadata_thread_id_type_2

        metadata_thread_id = _parse_metadata_thread_id(d.pop("metadata.thread_id", UNSET))

        def _parse_metadata_customer_id(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersMetadataCustomerIdType1
            | PostApiTracesSearchBodyFiltersMetadataCustomerIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_customer_id_type_0 = cast(list[str], data)

                return metadata_customer_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_customer_id_type_1 = PostApiTracesSearchBodyFiltersMetadataCustomerIdType1.from_dict(data)

                return metadata_customer_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_customer_id_type_2 = PostApiTracesSearchBodyFiltersMetadataCustomerIdType2.from_dict(data)

            return metadata_customer_id_type_2

        metadata_customer_id = _parse_metadata_customer_id(d.pop("metadata.customer_id", UNSET))

        def _parse_metadata_labels(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersMetadataLabelsType1
            | PostApiTracesSearchBodyFiltersMetadataLabelsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_labels_type_0 = cast(list[str], data)

                return metadata_labels_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_labels_type_1 = PostApiTracesSearchBodyFiltersMetadataLabelsType1.from_dict(data)

                return metadata_labels_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_labels_type_2 = PostApiTracesSearchBodyFiltersMetadataLabelsType2.from_dict(data)

            return metadata_labels_type_2

        metadata_labels = _parse_metadata_labels(d.pop("metadata.labels", UNSET))

        def _parse_metadata_key(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersMetadataKeyType1
            | PostApiTracesSearchBodyFiltersMetadataKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_key_type_0 = cast(list[str], data)

                return metadata_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_key_type_1 = PostApiTracesSearchBodyFiltersMetadataKeyType1.from_dict(data)

                return metadata_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_key_type_2 = PostApiTracesSearchBodyFiltersMetadataKeyType2.from_dict(data)

            return metadata_key_type_2

        metadata_key = _parse_metadata_key(d.pop("metadata.key", UNSET))

        def _parse_metadata_value(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersMetadataValueType1
            | PostApiTracesSearchBodyFiltersMetadataValueType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_value_type_0 = cast(list[str], data)

                return metadata_value_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_value_type_1 = PostApiTracesSearchBodyFiltersMetadataValueType1.from_dict(data)

                return metadata_value_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_value_type_2 = PostApiTracesSearchBodyFiltersMetadataValueType2.from_dict(data)

            return metadata_value_type_2

        metadata_value = _parse_metadata_value(d.pop("metadata.value", UNSET))

        def _parse_metadata_prompt_ids(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersMetadataPromptIdsType1
            | PostApiTracesSearchBodyFiltersMetadataPromptIdsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_prompt_ids_type_0 = cast(list[str], data)

                return metadata_prompt_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_prompt_ids_type_1 = PostApiTracesSearchBodyFiltersMetadataPromptIdsType1.from_dict(data)

                return metadata_prompt_ids_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_prompt_ids_type_2 = PostApiTracesSearchBodyFiltersMetadataPromptIdsType2.from_dict(data)

            return metadata_prompt_ids_type_2

        metadata_prompt_ids = _parse_metadata_prompt_ids(d.pop("metadata.prompt_ids", UNSET))

        def _parse_traces_error(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersTracesErrorType1
            | PostApiTracesSearchBodyFiltersTracesErrorType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                traces_error_type_0 = cast(list[str], data)

                return traces_error_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                traces_error_type_1 = PostApiTracesSearchBodyFiltersTracesErrorType1.from_dict(data)

                return traces_error_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            traces_error_type_2 = PostApiTracesSearchBodyFiltersTracesErrorType2.from_dict(data)

            return traces_error_type_2

        traces_error = _parse_traces_error(d.pop("traces.error", UNSET))

        def _parse_spans_type(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersSpansTypeType1
            | PostApiTracesSearchBodyFiltersSpansTypeType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                spans_type_type_0 = cast(list[str], data)

                return spans_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                spans_type_type_1 = PostApiTracesSearchBodyFiltersSpansTypeType1.from_dict(data)

                return spans_type_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            spans_type_type_2 = PostApiTracesSearchBodyFiltersSpansTypeType2.from_dict(data)

            return spans_type_type_2

        spans_type = _parse_spans_type(d.pop("spans.type", UNSET))

        def _parse_spans_model(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersSpansModelType1
            | PostApiTracesSearchBodyFiltersSpansModelType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                spans_model_type_0 = cast(list[str], data)

                return spans_model_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                spans_model_type_1 = PostApiTracesSearchBodyFiltersSpansModelType1.from_dict(data)

                return spans_model_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            spans_model_type_2 = PostApiTracesSearchBodyFiltersSpansModelType2.from_dict(data)

            return spans_model_type_2

        spans_model = _parse_spans_model(d.pop("spans.model", UNSET))

        def _parse_evaluations_evaluator_id(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType1
            | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_evaluator_id_type_0 = cast(list[str], data)

                return evaluations_evaluator_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_evaluator_id_type_1 = PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType1.from_dict(
                    data
                )

                return evaluations_evaluator_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_evaluator_id_type_2 = PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdType2.from_dict(data)

            return evaluations_evaluator_id_type_2

        evaluations_evaluator_id = _parse_evaluations_evaluator_id(d.pop("evaluations.evaluator_id", UNSET))

        def _parse_evaluations_evaluator_id_guardrails_only(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1
            | PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_evaluator_id_guardrails_only_type_0 = cast(list[str], data)

                return evaluations_evaluator_id_guardrails_only_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_evaluator_id_guardrails_only_type_1 = (
                    PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1.from_dict(data)
                )

                return evaluations_evaluator_id_guardrails_only_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_evaluator_id_guardrails_only_type_2 = (
                PostApiTracesSearchBodyFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2.from_dict(data)
            )

            return evaluations_evaluator_id_guardrails_only_type_2

        evaluations_evaluator_id_guardrails_only = _parse_evaluations_evaluator_id_guardrails_only(
            d.pop("evaluations.evaluator_id.guardrails_only", UNSET)
        )

        def _parse_evaluations_passed(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEvaluationsPassedType1
            | PostApiTracesSearchBodyFiltersEvaluationsPassedType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_passed_type_0 = cast(list[str], data)

                return evaluations_passed_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_passed_type_1 = PostApiTracesSearchBodyFiltersEvaluationsPassedType1.from_dict(data)

                return evaluations_passed_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_passed_type_2 = PostApiTracesSearchBodyFiltersEvaluationsPassedType2.from_dict(data)

            return evaluations_passed_type_2

        evaluations_passed = _parse_evaluations_passed(d.pop("evaluations.passed", UNSET))

        def _parse_evaluations_score(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEvaluationsScoreType1
            | PostApiTracesSearchBodyFiltersEvaluationsScoreType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_score_type_0 = cast(list[str], data)

                return evaluations_score_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_score_type_1 = PostApiTracesSearchBodyFiltersEvaluationsScoreType1.from_dict(data)

                return evaluations_score_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_score_type_2 = PostApiTracesSearchBodyFiltersEvaluationsScoreType2.from_dict(data)

            return evaluations_score_type_2

        evaluations_score = _parse_evaluations_score(d.pop("evaluations.score", UNSET))

        def _parse_evaluations_state(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEvaluationsStateType1
            | PostApiTracesSearchBodyFiltersEvaluationsStateType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_state_type_0 = cast(list[str], data)

                return evaluations_state_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_state_type_1 = PostApiTracesSearchBodyFiltersEvaluationsStateType1.from_dict(data)

                return evaluations_state_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_state_type_2 = PostApiTracesSearchBodyFiltersEvaluationsStateType2.from_dict(data)

            return evaluations_state_type_2

        evaluations_state = _parse_evaluations_state(d.pop("evaluations.state", UNSET))

        def _parse_evaluations_label(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEvaluationsLabelType1
            | PostApiTracesSearchBodyFiltersEvaluationsLabelType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_label_type_0 = cast(list[str], data)

                return evaluations_label_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_label_type_1 = PostApiTracesSearchBodyFiltersEvaluationsLabelType1.from_dict(data)

                return evaluations_label_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_label_type_2 = PostApiTracesSearchBodyFiltersEvaluationsLabelType2.from_dict(data)

            return evaluations_label_type_2

        evaluations_label = _parse_evaluations_label(d.pop("evaluations.label", UNSET))

        def _parse_events_event_type(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEventsEventTypeType1
            | PostApiTracesSearchBodyFiltersEventsEventTypeType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_event_type_type_0 = cast(list[str], data)

                return events_event_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_event_type_type_1 = PostApiTracesSearchBodyFiltersEventsEventTypeType1.from_dict(data)

                return events_event_type_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_event_type_type_2 = PostApiTracesSearchBodyFiltersEventsEventTypeType2.from_dict(data)

            return events_event_type_type_2

        events_event_type = _parse_events_event_type(d.pop("events.event_type", UNSET))

        def _parse_events_metrics_key(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEventsMetricsKeyType1
            | PostApiTracesSearchBodyFiltersEventsMetricsKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_metrics_key_type_0 = cast(list[str], data)

                return events_metrics_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_metrics_key_type_1 = PostApiTracesSearchBodyFiltersEventsMetricsKeyType1.from_dict(data)

                return events_metrics_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_metrics_key_type_2 = PostApiTracesSearchBodyFiltersEventsMetricsKeyType2.from_dict(data)

            return events_metrics_key_type_2

        events_metrics_key = _parse_events_metrics_key(d.pop("events.metrics.key", UNSET))

        def _parse_events_metrics_value(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEventsMetricsValueType1
            | PostApiTracesSearchBodyFiltersEventsMetricsValueType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_metrics_value_type_0 = cast(list[str], data)

                return events_metrics_value_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_metrics_value_type_1 = PostApiTracesSearchBodyFiltersEventsMetricsValueType1.from_dict(data)

                return events_metrics_value_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_metrics_value_type_2 = PostApiTracesSearchBodyFiltersEventsMetricsValueType2.from_dict(data)

            return events_metrics_value_type_2

        events_metrics_value = _parse_events_metrics_value(d.pop("events.metrics.value", UNSET))

        def _parse_events_event_details_key(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType1
            | PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_event_details_key_type_0 = cast(list[str], data)

                return events_event_details_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_event_details_key_type_1 = PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType1.from_dict(
                    data
                )

                return events_event_details_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_event_details_key_type_2 = PostApiTracesSearchBodyFiltersEventsEventDetailsKeyType2.from_dict(data)

            return events_event_details_key_type_2

        events_event_details_key = _parse_events_event_details_key(d.pop("events.event_details.key", UNSET))

        def _parse_annotations_has_annotation(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType1
            | PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                annotations_has_annotation_type_0 = cast(list[str], data)

                return annotations_has_annotation_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                annotations_has_annotation_type_1 = (
                    PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType1.from_dict(data)
                )

                return annotations_has_annotation_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            annotations_has_annotation_type_2 = PostApiTracesSearchBodyFiltersAnnotationsHasAnnotationType2.from_dict(
                data
            )

            return annotations_has_annotation_type_2

        annotations_has_annotation = _parse_annotations_has_annotation(d.pop("annotations.hasAnnotation", UNSET))

        def _parse_sentiment_input_sentiment(
            data: object,
        ) -> (
            list[str]
            | PostApiTracesSearchBodyFiltersSentimentInputSentimentType1
            | PostApiTracesSearchBodyFiltersSentimentInputSentimentType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                sentiment_input_sentiment_type_0 = cast(list[str], data)

                return sentiment_input_sentiment_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sentiment_input_sentiment_type_1 = PostApiTracesSearchBodyFiltersSentimentInputSentimentType1.from_dict(
                    data
                )

                return sentiment_input_sentiment_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            sentiment_input_sentiment_type_2 = PostApiTracesSearchBodyFiltersSentimentInputSentimentType2.from_dict(
                data
            )

            return sentiment_input_sentiment_type_2

        sentiment_input_sentiment = _parse_sentiment_input_sentiment(d.pop("sentiment.input_sentiment", UNSET))

        post_api_traces_search_body_filters = cls(
            topics_topics=topics_topics,
            topics_subtopics=topics_subtopics,
            metadata_user_id=metadata_user_id,
            metadata_thread_id=metadata_thread_id,
            metadata_customer_id=metadata_customer_id,
            metadata_labels=metadata_labels,
            metadata_key=metadata_key,
            metadata_value=metadata_value,
            metadata_prompt_ids=metadata_prompt_ids,
            traces_error=traces_error,
            spans_type=spans_type,
            spans_model=spans_model,
            evaluations_evaluator_id=evaluations_evaluator_id,
            evaluations_evaluator_id_guardrails_only=evaluations_evaluator_id_guardrails_only,
            evaluations_passed=evaluations_passed,
            evaluations_score=evaluations_score,
            evaluations_state=evaluations_state,
            evaluations_label=evaluations_label,
            events_event_type=events_event_type,
            events_metrics_key=events_metrics_key,
            events_metrics_value=events_metrics_value,
            events_event_details_key=events_event_details_key,
            annotations_has_annotation=annotations_has_annotation,
            sentiment_input_sentiment=sentiment_input_sentiment,
        )

        return post_api_traces_search_body_filters



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_api_dataset_by_slug_or_id_response_200 import GetApiDatasetBySlugOrIdResponse200
from ...models.get_api_dataset_by_slug_or_id_response_400 import GetApiDatasetBySlugOrIdResponse400
from ...models.get_api_dataset_by_slug_or_id_response_401 import GetApiDatasetBySlugOrIdResponse401
from ...models.get_api_dataset_by_slug_or_id_response_404 import GetApiDatasetBySlugOrIdResponse404
from ...models.get_api_dataset_by_slug_or_id_response_422 import GetApiDatasetBySlugOrIdResponse422
from ...models.get_api_dataset_by_slug_or_id_response_500 import GetApiDatasetBySlugOrIdResponse500
from ...types import Response


def _get_kwargs(
    slug_or_id: str,
) -> dict[str, Any]:

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/dataset/{slug_or_id}".format(
            slug_or_id=quote(str(slug_or_id), safe=""),
        ),
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    GetApiDatasetBySlugOrIdResponse200
    | GetApiDatasetBySlugOrIdResponse400
    | GetApiDatasetBySlugOrIdResponse401
    | GetApiDatasetBySlugOrIdResponse404
    | GetApiDatasetBySlugOrIdResponse422
    | GetApiDatasetBySlugOrIdResponse500
    | None
):
    if response.status_code == 200:
        response_200 = GetApiDatasetBySlugOrIdResponse200.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = GetApiDatasetBySlugOrIdResponse400.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = GetApiDatasetBySlugOrIdResponse401.from_dict(response.json())

        return response_401

    if response.status_code == 404:
        response_404 = GetApiDatasetBySlugOrIdResponse404.from_dict(response.json())

        return response_404

    if response.status_code == 422:
        response_422 = GetApiDatasetBySlugOrIdResponse422.from_dict(response.json())

        return response_422

    if response.status_code == 500:
        response_500 = GetApiDatasetBySlugOrIdResponse500.from_dict(response.json())

        return response_500

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    GetApiDatasetBySlugOrIdResponse200
    | GetApiDatasetBySlugOrIdResponse400
    | GetApiDatasetBySlugOrIdResponse401
    | GetApiDatasetBySlugOrIdResponse404
    | GetApiDatasetBySlugOrIdResponse422
    | GetApiDatasetBySlugOrIdResponse500
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    slug_or_id: str,
    *,
    client: AuthenticatedClient | Client,
) -> Response[
    GetApiDatasetBySlugOrIdResponse200
    | GetApiDatasetBySlugOrIdResponse400
    | GetApiDatasetBySlugOrIdResponse401
    | GetApiDatasetBySlugOrIdResponse404
    | GetApiDatasetBySlugOrIdResponse422
    | GetApiDatasetBySlugOrIdResponse500
]:
    """Get a dataset by its slug or id.

    Args:
        slug_or_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GetApiDatasetBySlugOrIdResponse200 | GetApiDatasetBySlugOrIdResponse400 | GetApiDatasetBySlugOrIdResponse401 | GetApiDatasetBySlugOrIdResponse404 | GetApiDatasetBySlugOrIdResponse422 | GetApiDatasetBySlugOrIdResponse500]
    """

    kwargs = _get_kwargs(
        slug_or_id=slug_or_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    slug_or_id: str,
    *,
    client: AuthenticatedClient | Client,
) -> (
    GetApiDatasetBySlugOrIdResponse200
    | GetApiDatasetBySlugOrIdResponse400
    | GetApiDatasetBySlugOrIdResponse401
    | GetApiDatasetBySlugOrIdResponse404
    | GetApiDatasetBySlugOrIdResponse422
    | GetApiDatasetBySlugOrIdResponse500
    | None
):
    """Get a dataset by its slug or id.

    Args:
        slug_or_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GetApiDatasetBySlugOrIdResponse200 | GetApiDatasetBySlugOrIdResponse400 | GetApiDatasetBySlugOrIdResponse401 | GetApiDatasetBySlugOrIdResponse404 | GetApiDatasetBySlugOrIdResponse422 | GetApiDatasetBySlugOrIdResponse500
    """

    return sync_detailed(
        slug_or_id=slug_or_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    slug_or_id: str,
    *,
    client: AuthenticatedClient | Client,
) -> Response[
    GetApiDatasetBySlugOrIdResponse200
    | GetApiDatasetBySlugOrIdResponse400
    | GetApiDatasetBySlugOrIdResponse401
    | GetApiDatasetBySlugOrIdResponse404
    | GetApiDatasetBySlugOrIdResponse422
    | GetApiDatasetBySlugOrIdResponse500
]:
    """Get a dataset by its slug or id.

    Args:
        slug_or_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GetApiDatasetBySlugOrIdResponse200 | GetApiDatasetBySlugOrIdResponse400 | GetApiDatasetBySlugOrIdResponse401 | GetApiDatasetBySlugOrIdResponse404 | GetApiDatasetBySlugOrIdResponse422 | GetApiDatasetBySlugOrIdResponse500]
    """

    kwargs = _get_kwargs(
        slug_or_id=slug_or_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    slug_or_id: str,
    *,
    client: AuthenticatedClient | Client,
) -> (
    GetApiDatasetBySlugOrIdResponse200
    | GetApiDatasetBySlugOrIdResponse400
    | GetApiDatasetBySlugOrIdResponse401
    | GetApiDatasetBySlugOrIdResponse404
    | GetApiDatasetBySlugOrIdResponse422
    | GetApiDatasetBySlugOrIdResponse500
    | None
):
    """Get a dataset by its slug or id.

    Args:
        slug_or_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GetApiDatasetBySlugOrIdResponse200 | GetApiDatasetBySlugOrIdResponse400 | GetApiDatasetBySlugOrIdResponse401 | GetApiDatasetBySlugOrIdResponse404 | GetApiDatasetBySlugOrIdResponse422 | GetApiDatasetBySlugOrIdResponse500
    """

    return (
        await asyncio_detailed(
            slug_or_id=slug_or_id,
            client=client,
        )
    ).parsed
